Crate diskann_vector

Crate diskann_vector 

vector

This crate contains SIMD accelerated functions for operating on vector data. Note that the name ‘vector’ does not exclusively mean embedding vectors, but any array of data appropriate for SIMD. Therefor, aside from fast implementations of distance for real vectors, this crate also includes things like SIMD accelerated contains for slices.

Modules

contains
conversion
distance
norm

Structs

MathematicalValue
The DiskANN library uses f32 values to represent similarity scores, but distance functions can return either similarity-scores (those that are potentially transformed so that minimization yields higher similarity) and mathematical values (those that are computed from the mathematical definition of the operation.
SimilarityScore
The DiskANN library uses f32 values to represent similarity scores, but distance functions can return either similarity-scores (those that are potentially transformed so that minimization yields higher similarity) and mathematical values (those that are computed from the mathematical definition of the operation.

Traits

DistanceFunction
An overloadable, 2-argument distance function with a parameterized return type.
DistanceFunctionMut
A mutable version of DistanceFunction that allows mutation of Self.
Norm
Evaluate a norm of the argument x and return the result as the requested type.
PreprocessedDistanceFunction
A distance function where one argument is static and interned within Self.
PureDistanceFunction
An overloadable, 2-argument distance function with a parameterized return type.

Functions

prefetch_hint_all
prefetch_hint_max

Type Aliases

Half