Module conversion

Module conversion 

Structs

SliceCast
A zero-sized type providing implementations of diskann_wide::arch::Target2 to provide platform-dependent conversions between slices of the two generic types.

Traits

CastFromSlice
Perform a numeric cast on a slice of values.