Specialize

Struct Specialize 

pub struct Specialize<const N: usize, F>(/* private fields */);
A utility for specializing distance computations for fixed-length slices.

When F implements diskann_wide::arch::Target2 for UnalignedSlice, Specialize<N, F> will implement diskann_wide::arch::FTarget2 for slices of exactly length N. This can lead to must better generated code, especially when N is small.

Passing slices of length not equal to N will panic.

use diskann_vector::{AsUnaligned, distance::{Specialize, SquaredL2}};
use diskann_wide::{ARCH, arch::FTarget2};

let x = [1.0f32, 2.0, 3.0];
let y = [2.0f32, 3.0, 4.0];

let result: f32 = Specialize::<3, SquaredL2>::run(
    ARCH,
    x.as_unaligned(),
    y.as_unaligned(),
);

assert_eq!(result, 3.0);

Implementations

Source

impl<const N: usize, F> Specialize<N, F>

pub const fn new() -> Self

Create a new Specialize.