pub struct FullCosineAccumulator<T> { /* private fields */ }Expand description
Accumulator of partial products for a full cosine distance computation (where the norms of both the query and the dataset vector are computed on the fly).
Implementations
Sourceimpl<T> FullCosineAccumulator<T>where
T: SIMDVector + SIMDSumTree + SIMDMulAdd + Mul<Output = T> + Add<Output = T>,
T::Scalar: LossyF32Conversion,
impl<T> FullCosineAccumulator<T>where
T: SIMDVector + SIMDSumTree + SIMDMulAdd + Mul<Output = T> + Add<Output = T>,
T::Scalar: LossyF32Conversion,