Loader

Struct Loader 

pub struct Loader<Schema, Left, Right, A>
where Schema: SIMDSchema<Left, Right, A>, A: Architecture,
{ /* private fields */ }
A utility struct to help with SIMD loading.

The main loop of SIMD kernels consists of various tilings of loads and arithmetic. Outside of the epilogue, these loads are all full-width vector loads.

To aid in defining different tilings, this struct takes the base pointers for left and right hand pointers and provides a load method to extract full vectors for both the left and right-hand sides.

This works in conjunction with the SIMDSchema to help write unrolled loops.