pub struct MathematicalValue<T>(/* private fields */)
where
T: Copy;Expand description
The DiskANN library uses f32 values to represent similarity scores, but distance
functions can return either similarity-scores (those that are potentially transformed
so that minimization yields higher similarity) and mathematical values (those that
are computed from the mathematical definition of the operation.
Since those are currently mixed in the library with the more common use being a
similarity-score, make the MathematicalValue the type to represent no transformation
was performed.
Implementations
Sourceimpl<T: Copy> MathematicalValue<T>
impl<T: Copy> MathematicalValue<T>
pub fn new(value: T) -> Self
pub fn into_inner(self) -> T