pub trait Norm<T, To = f32> {
// Required method
fn evaluate(&self, x: T) -> To;
}Expand description
Evaluate a norm of the argument x and return the result as the requested type.
Note that while this has a similar signature to PreprocessedDistanceFunction, the
semantics of this trait are different. Implementations are expected to be light-weight
types that implement some kind of reduction on on x.
Required Methods
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".