pub trait PureDistanceFunction<Left, Right, To = f32> {
// Required method
fn evaluate(x: Left, y: Right) -> To;
}Expand description
An overloadable, 2-argument distance function with a parameterized return type.
Pure distance functions depend only on the values of the argument and the type of the return value.
Required Methods
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.