vector
This crate contains SIMD accelerated functions for operating on vector data. Note that the name ‘vector’ does not exclusively mean embedding vectors, but any array of data appropriate for SIMD. Therefor, aside from fast implementations of distance for real vectors, this crate also includes things like SIMD accelerated contains for slices.
Modules
Structs
- Mathematical
Value - The DiskANN library uses
f32values to represent similarity scores, but distance functions can return either similarity-scores (those that are potentially transformed so that minimization yields higher similarity) and mathematical values (those that are computed from the mathematical definition of the operation. - Similarity
Score - The DiskANN library uses
f32values to represent similarity scores, but distance functions can return either similarity-scores (those that are potentially transformed so that minimization yields higher similarity) and mathematical values (those that are computed from the mathematical definition of the operation.
Traits
- Distance
Function - An overloadable, 2-argument distance function with a parameterized return type.
- Distance
Function Mut - A mutable version of
DistanceFunctionthat allows mutation ofSelf. - Norm
- Evaluate a norm of the argument
xand return the result as the requested type. - Preprocessed
Distance Function - A distance function where one argument is static and interned within
Self. - Pure
Distance Function - An overloadable, 2-argument distance function with a parameterized return type.