pub struct Distance<T, U>where
T: 'static,
U: 'static,{ /* private fields */ }Expand description
A function pointer-like type for computing distances between &[T] and &[U].
See: DistanceProvider.
Implementations
Sourceimpl<T, U> Distance<T, U>where
T: 'static,
U: 'static,
impl<T, U> Distance<T, U>where
T: 'static,
U: 'static,
Sourcepub fn call(&self, x: &[T], y: &[U]) -> f32
pub fn call(&self, x: &[T], y: &[U]) -> f32
Compute the distance between x and y.
The actual distance computed depends on the metric supplied to DistanceProvider.
Additionally, if a dimension were given to DistanceProvider, this function may
panic if provided with slices with a length not equal to this dimension.
Sourcepub fn call_unaligned(
&self,
x: UnalignedSlice<'_, T>,
y: UnalignedSlice<'_, U>,
) -> f32
pub fn call_unaligned( &self, x: UnalignedSlice<'_, T>, y: UnalignedSlice<'_, U>, ) -> f32
Compute the distance between x and y.
The actual distance computed depends on the metric supplied to DistanceProvider.
Additionally, if a dimension were given to DistanceProvider, this function may
panic if provided with slices with a length not equal to this dimension.