pub struct FastL2Norm;Expand description
Evaluate the L2 norm of the argument.
Implementation
The implementations behind this method use a naive approach to computing the norm. This is faster but less accurate than more precise methods.